/*
	Base components, customized for WAPI server usage
*/

/*
	Define required extjs modules
*/
Ext.require(['Ext.data.*', 'Ext.grid.*']);

/*
	Base proxy for ajax calls to WAPI server. Supports CRUD operations.
	Sends requests in next format (WAPI servers expects it):
	
	- GET wapi/objtype                                            load list of 'objtype', for ex: wapi/network returns list of networks
	- POST wapi/objtype with body: {field1: '...', ...}           create object of 'objtype' type with appropriatelly filled properties 
	- PUT wapi/objref with body: {field1: '...', ...}             update object with id == 'objref' with new values in appropriate fields
	- DELETE wapi/objref                                          delete object with id == 'objref'
*/
Ext.define('BaseProxy', {
	extend:'Ext.data.proxy.Rest',                                 // WAPI server is REST server     
	
	noCache: false, limitParam: undefined,                        // Remove all default parametres for GET requests
	startParam: undefined, pageParam: undefined, 
	
	headers: {'Accept': 'application/json', 'Cache-Control' : 'no-cache, no-store'},                      // Supported response format - JSON
	reader: { type: 'json'},
    writer: { type: 'json', writeAllFields: false},
	
	alias : 'proxy.baseproxy',                                    // Alias for Ext.create method
	
	url:'/wapi/v1.0',                                                   // Base URL 
	
	// build url for appropriate request.
	// 'read' and 'create' operations require 'objtype' parameter in URL
	buildUrl: function(request){
		var url = this.callParent(arguments);
		if (request.action === 'read' || request.action === 'create'){
			url = url + '/' + this.model.modelName.toLowerCase();
		} 
		request.url = url;
		return url;
	}
});


/*
	Base Model - represents any object, contains all common properties/methods.
	Not designed for instantiation - only for extention
*/
Ext.define('BaseObject', {
	extend: 'Ext.data.Model',
	// all objects has '_ref' property as unique key
	idProperty: '_ref',
    fields: [{name: '_ref'}],
	// return true if current object haven't been yet saved in server
	isNew: function(){
		return Ext.isEmpty(this.get('_ref'));
	},
	// return true if current object can be edited
	isEditable: function(){
		return true;
	}
});

/* 
	Network view object model
*/
Ext.define('NetworkView', {
    extend: 'BaseObject',
	//  network view fields
	fields: [
		'name', 
		// do no persist this propety - server calculates it itself
		{name: 'is_default', persist: false}, 
		'comment'
	],
	// object validation: 'name' is required, max length for 'name' and 'comment' - 256 symbols
	validations: [
		{type: 'presence',  field: 'name'},
		{type: 'length',    field: 'name',  max: 256},
		{type: 'length',    field: 'comment',  max: 256}
		
    ],
	// default network view is immutable object
	isEditable: function(){
		return !this.get('is_default');
	},
	// proxy for all CRUD operations
	proxy:{
		type:'baseproxy'
	}
});


/* 
	Network object model
*/
Ext.define('Network', {
    extend: 'BaseObject',
    // network fields
	fields: ['network', 'network_view', 'comment'],
	// validation - all fields, except 'comment' are required. 'comment' max length is 256 symbols
	validations: [
		{type: 'presence',  field: 'network'},
		{type: 'presence',  field: 'network_view'},
		{type: 'length',    field: 'comment',  max: 256}
    ],
	// proxy for all CRUD operations
	proxy:{
		type:'baseproxy'
	}
});

/*
 Network Container model
*/
Ext.define('NetworkContainer', {
    extend: 'BaseObject',
    // network container fields
	fields: ['auto_create_reversezone', 'comment', 
		'network', 'network_container', 'network_view', 'use_zone_associations'],
	// proxy for all CRUD operations
	proxy:{
		type:'baseproxy'
	}
});

/*
	Dhcp Range model
*/
Ext.define('Range', {
    extend: 'BaseObject',
    // range fields
	fields: ['start_addr', 'end_addr', 'name', 'disable', 
		'network', 'comment'],
	// proxy for all CRUD operations
	proxy:{
		type:'baseproxy'
	}
});

/*
	Grid for any object, extended from BaseObject
	- displays all items of some type
	- provides possibility to create/update/delete item 
*/
Ext.define('BaseGrid', {
	extend:'Ext.grid.Panel',
	
	alias: 'grid.basegrid', 

	initComponent: function(){
		// create editor for object update
		this.editor = Ext.create('Ext.grid.plugin.RowEditing');
		this.plugins = [this.editor];
		// create 'store' - component that hides all server operations
		this.store = Ext.create('Ext.data.Store',{
			autoLoad: true,
			autoSync: false,
			model: this.model
		});
		// create 2 toolbar items - 'add' and 'delete' buttons
		this.dockedItems = [{
            xtype: 'toolbar',
            items: [{
                text: 'Add',
                iconCls: 'icon-add',
                handler: Ext.Function.bind(this.addHandler, this)
            }, '-', {
                itemId: 'delete',
                text: 'Delete',
                iconCls: 'icon-delete',
                disabled: true,
                handler: Ext.Function.bind(this.deleteHandler, this)
            }]
        }]
		this.callParent(arguments);
		
		this.addEvents('operationperformed');
		
		this.initEventListeners();
		
	},
	// iniyialize all required event listeners for 'link' different grid parts together
	initEventListeners: function(){
		
		this.getStore().getProxy().on('exception', this.onException, this);
		this.getStore().on('write', this.onOperation, this);
		
		this.getSelectionModel().on('selectionchange', this.onSelectionChange, this);
		
		this.on('beforeedit', this.onBeforeEdit, this);
		this.on('canceledit', this.onCancelEdit, this);
		this.on('edit', this.onAfterEdit, this);
	},
	
	// update grid (reload items) after any performed operation (create/update/delete)
	onOperation: function(store, operation, eOpts ){
		store.reload();
		this.fireEvent('operationperformed', operation.action);
	},
	
	// show any server error to users
	onException: function(proxy, response, operation, eOpts){
		Ext.MessageBox.alert('Error', Ext.decode(response.responseText).text);		
		this.getStore().reload();
	},
	
	// after 'add' click - insert new object in grid and launch editor for it
	addHandler: function(){
		this.store.insert(this.store.count(), Ext.create(this.model));
        this.editor.startEdit(this.store.count() - 1, 0);
		this.fireEvent('objadded');
	},  
	// after 'delete' click - remove selected object
	deleteHandler: function(){
		var selection = this.getView().getSelectionModel().getSelection()[0];
        if (selection) {
            this.getStore().remove(selection);
			this.getStore().sync();
        }	
	},
	// update 'delete' button after selection change in grid.
	// 'delete' button is available only if user selected some object AND this object is 'editable'
	onSelectionChange: function(selModel, selections){
		var disabled = selections.length === 0;
		if (!disabled){
			Ext.each(selections, function(rec){
				disabled = disabled || !rec.isEditable();
			});
		}
		this.down('#delete').setDisabled(disabled);
	},
	
	// do not launch editor if selected object isn't editable
	// returb 'false' in this method will interrupt eitor launching
	onBeforeEdit: function(editor, e){ 
		return e.record.isEditable();
	},
	
	// save object (create new or update existing) after user clicks 'update' in editor
	onAfterEdit: function(editor, e){
		var errors = e.record.validate();
		if (!errors.isValid()){
			Ext.MessageBox.alert('Validation', '"' + errors.first().field + '" ' + errors.first().message);	
			this.getStore().reload();
		} else {
			this.getStore().sync();
		}
	},
	
	// if user cancels 'new object creation' - remove this 'empty' object from grid
	onCancelEdit: function(editor, e){
		if (e.record.isNew()) {
			e.store.remove(e.record);
		}
	}
});


/*
	Grid for any object, extended from BaseObject
	- displays all items of some type
	- provides possibility to create/update/delete item 
*/
Ext.define('SimpleGrid', {
	extend:'Ext.grid.Panel',
	
	alias: 'grid.simplegrid', 

	initComponent: function(){
		// create editor for object update
		// create 'store' - component that hides all server operations
		this.store = Ext.create('Ext.data.Store',{
			autoLoad: true,
			autoSync: false,
			model: this.model
		});
		this.callParent(arguments);
		
		this.initEventListeners();
		
	},
	// iniyialize all required event listeners for 'link' different grid parts together
	initEventListeners: function(){
		this.getStore().getProxy().on('exception', this.onException, this);
	},
	
	// show any server error to users
	onException: function(proxy, response, operation, eOpts){
		Ext.MessageBox.alert('Error', Ext.decode(response.responseText).text);		
		this.getStore().reload();
	}
});